@@ -3,8 +3,18 @@ module Test.Hspec.Core.Config.Definition (
   Config(..)
 , ColorMode(..)
 , UnicodeMode(..)
+, mkDefaultConfig
+
 , filterOr
-, defaultConfig
+
+, setConfigAnnotation
+, getConfigAnnotation
+
+, addExtensionOptions
+, getExtensionOptions
+
+, getSeed
+, getFormatter
 
 , commandLineOnlyOptions
 , formatterOptions
@@ -12,6 +22,9 @@ module Test.Hspec.Core.Config.Definition (
 , quickCheckOptions
 , runnerOptions
 
+, flag
+, option
+
 #ifdef TEST
 , splitOn
 #endif
@@ -20,20 +33,43 @@ module Test.Hspec.Core.Config.Definition (
 import           Prelude ()
 import           Test.Hspec.Core.Compat
 
-import           Control.Exception (bracket)
 import           System.Directory (getTemporaryDirectory, removeFile)
 import           System.IO (openTempFile, hClose)
 import           System.Process (system)
 
-import           Test.Hspec.Core.Example (Params(..), defaultParams)
+import           Test.Hspec.Core.Annotations (Annotations)
+import qualified Test.Hspec.Core.Annotations as Annotations
+
 import           Test.Hspec.Core.Format (Format, FormatConfig)
 import           Test.Hspec.Core.Formatters.Pretty (pretty2)
-import qualified Test.Hspec.Core.Formatters.V1 as V1
-import qualified Test.Hspec.Core.Formatters.V2 as V2
+import qualified Test.Hspec.Core.Formatters.V1.Monad as V1
+import qualified Test.Hspec.Core.Formatters.V1.Internal as V1 (formatterToFormat)
 import           Test.Hspec.Core.Util
 
 import           GetOpt.Declarative
 
+setConfigAnnotation :: Typeable value => value -> Config -> Config
+setConfigAnnotation value config = config { configAnnotations = Annotations.setValue value $ configAnnotations config }
+
+getConfigAnnotation :: Typeable value => Config -> Maybe value
+getConfigAnnotation = Annotations.getValue . configAnnotations
+
+newtype ExtensionOptions = ExtensionOptions { unExtensionOptions :: [(String, [Option Config])] }
+
+addExtensionOptions :: String -> [Option Config] -> Config -> Config
+addExtensionOptions section opts config = setExtensionOptions ((section, opts) : getExtensionOptions config) config
+
+setExtensionOptions :: [(String, [Option Config])] -> Config -> Config
+setExtensionOptions = setConfigAnnotation . ExtensionOptions
+
+getExtensionOptions :: Config -> [(String, [Option Config])]
+getExtensionOptions = maybe [] unExtensionOptions . getConfigAnnotation
+
+getFormatter :: Config -> Maybe (FormatConfig -> IO Format)
+getFormatter config = configFormat config <|> V1.formatterToFormat <$> configFormatter config
+
+getSeed :: Config -> Maybe Integer
+getSeed config = configSeed config <|> configQuickCheckSeed config
 
 data ColorMode = ColorAuto | ColorNever | ColorAlways
   deriving (Eq, Show)
@@ -48,10 +84,13 @@ data Config = Config {
 , configFailOnEmpty :: Bool
 , configFailOnFocused :: Bool
 , configFailOnPending :: Bool
+, configFailOnEmptyDescription :: Bool
 , configPrintSlowItems :: Maybe Int
 , configPrintCpuTime :: Bool
 , configFailFast :: Bool
 , configRandomize :: Bool
+, configSeed :: Maybe Integer
+, configQuickCheckSeed :: Maybe Integer
 , configFailureReport :: Maybe FilePath
 , configRerun :: Bool
 , configRerunAllOnSuccess :: Bool
@@ -61,7 +100,6 @@ data Config = Config {
 -- that satisfy the predicate are run.
 , configFilterPredicate :: Maybe (Path -> Bool)
 , configSkipPredicate :: Maybe (Path -> Bool)
-, configQuickCheckSeed :: Maybe Integer
 , configQuickCheckMaxSuccess :: Maybe Int
 , configQuickCheckMaxDiscardRatio :: Maybe Int
 , configQuickCheckMaxSize :: Maybe Int
@@ -82,37 +120,44 @@ data Config = Config {
 
 , configPrettyPrint :: Bool
 , configPrettyPrintFunction :: Bool -> String -> String -> (String, String)
+, configFormatException :: SomeException -> String -- ^ @since 2.11.5
 , configTimes :: Bool
-, configAvailableFormatters :: [(String, FormatConfig -> IO Format)]
+, configExpertMode :: Bool -- ^ @since 2.11.2
+, configAvailableFormatters :: [(String, FormatConfig -> IO Format)] -- ^ @since 2.9.0
 , configFormat :: Maybe (FormatConfig -> IO Format)
-, configFormatter :: Maybe V1.Formatter -- ^ deprecated, use `configFormat` instead
+, configFormatter :: Maybe V1.Formatter
 , configHtmlOutput :: Bool
 , configConcurrentJobs :: Maybe Int
+, configAnnotations :: Annotations
 }
+{-# DEPRECATED configFormatter "Use [@useFormatter@](https://hackage.haskell.org/package/hspec-api/docs/Test-Hspec-Api-Formatters-V1.html#v:useFormatter) instead." #-}
+{-# DEPRECATED configQuickCheckSeed "Use `configSeed` instead." #-}
 
-defaultConfig :: Config
-defaultConfig = Config {
+mkDefaultConfig :: [(String, FormatConfig -> IO Format)] -> Config
+mkDefaultConfig formatters = Config {
   configIgnoreConfigFile = False
 , configDryRun = False
 , configFocusedOnly = False
 , configFailOnEmpty = False
 , configFailOnFocused = False
 , configFailOnPending = False
+, configFailOnEmptyDescription = False
 , configPrintSlowItems = Nothing
 , configPrintCpuTime = False
 , configFailFast = False
 , configRandomize = False
+, configSeed = Nothing
+, configQuickCheckSeed = Nothing
 , configFailureReport = Nothing
 , configRerun = False
 , configRerunAllOnSuccess = False
 , configFilterPredicate = Nothing
 , configSkipPredicate = Nothing
-, configQuickCheckSeed = Nothing
 , configQuickCheckMaxSuccess = Nothing
 , configQuickCheckMaxDiscardRatio = Nothing
 , configQuickCheckMaxSize = Nothing
 , configQuickCheckMaxShrinks = Nothing
-, configSmallCheckDepth = paramsSmallCheckDepth defaultParams
+, configSmallCheckDepth = Nothing
 , configColorMode = ColorAuto
 , configUnicodeMode = UnicodeAuto
 , configDiff = True
@@ -120,18 +165,15 @@ defaultConfig = Config {
 , configExternalDiff = Nothing
 , configPrettyPrint = True
 , configPrettyPrintFunction = pretty2
+, configFormatException = formatExceptionWith show
 , configTimes = False
-, configAvailableFormatters = map (fmap V2.formatterToFormat) [
-    ("checks", V2.checks)
-  , ("specdoc", V2.specdoc)
-  , ("progress", V2.progress)
-  , ("failed-examples", V2.failed_examples)
-  , ("silent", V2.silent)
-  ]
+, configExpertMode = False
+, configAvailableFormatters = formatters
 , configFormat = Nothing
 , configFormatter = Nothing
 , configHtmlOutput = False
 , configConcurrentJobs = Nothing
+, configAnnotations = mempty
 }
 
 defaultDiffContext :: Int
@@ -154,8 +196,8 @@ withTempFile dir file contents action = do
 option :: String -> OptionSetter config -> String -> Option config
 option name arg help = Option name Nothing arg help True
 
-mkFlag :: String -> (Bool -> Config -> Config) -> String -> Option Config
-mkFlag name setter = option name (Flag setter)
+flag :: String -> (Bool -> config -> config) -> String -> Option config
+flag name setter = option name (Flag setter)
 
 mkOptionNoArg :: String -> Maybe Char -> (Config -> Config) -> String -> Option Config
 mkOptionNoArg name shortcut setter help = Option name shortcut (NoArg setter) help True
@@ -166,24 +208,28 @@ mkOption name shortcut arg help = Option name shortcut arg help True
 undocumented :: Option config -> Option config
 undocumented opt = opt {optionDocumented = False}
 
-argument :: String -> (String -> Maybe a) -> (a -> Config -> Config) -> OptionSetter Config
+argument :: String -> (String -> Maybe a) -> (a -> config -> config) -> OptionSetter config
 argument name parser setter = Arg name $ \ input c -> flip setter c <$> parser input
 
 formatterOptions :: [(String, FormatConfig -> IO Format)] -> [Option Config]
 formatterOptions formatters = [
     mkOption "format" (Just 'f') (argument "NAME" readFormatter setFormatter) helpForFormat
-  , mkFlag "color" setColor "colorize the output"
-  , mkFlag "unicode" setUnicode "output unicode"
-  , mkFlag "diff" setDiff "show colorized diffs"
+  , flag "color" setColor "colorize the output"
+  , flag "unicode" setUnicode "output unicode"
+  , flag "diff" setDiff "show colorized diffs"
   , option "diff-context" (argument "N" readDiffContext setDiffContext) $ unlines [
         "output N lines of diff context (default: " <> show defaultDiffContext <> ")"
       , "use a value of 'full' to see the full context"
       ]
   , option "diff-command" (argument "CMD" return setDiffCommand) "use an external diff command\nexample: --diff-command=\"git diff\""
-  , mkFlag "pretty" setPretty "try to pretty-print diff values"
-  , mkFlag "times" setTimes "report times for individual spec items"
+  , flag "pretty" setPretty "try to pretty-print diff values"
+  , mkOptionNoArg "show-exceptions" Nothing setShowException "use `show` when formatting exceptions"
+  , mkOptionNoArg "display-exceptions" Nothing setDisplayException "use `displayException` when formatting exceptions"
+
+  , flag "times" setTimes "report times for individual spec items"
   , mkOptionNoArg "print-cpu-time" Nothing setPrintCpuTime "include used CPU time in summary"
   , printSlowItemsOption
+  , flag "expert" setExpertMode "be less verbose"
 
     -- undocumented for now, as we probably want to change this to produce a
     -- standalone HTML report in the future
@@ -230,11 +276,20 @@ formatterOptions formatters = [
     setPretty :: Bool -> Config -> Config
     setPretty v config = config {configPrettyPrint = v}
 
+    setShowException :: Config -> Config
+    setShowException config = config {configFormatException = formatExceptionWith show}
+
+    setDisplayException :: Config -> Config
+    setDisplayException config = config {configFormatException = formatExceptionWith displayException}
+
     setTimes :: Bool -> Config -> Config
     setTimes v config = config {configTimes = v}
 
     setPrintCpuTime config = config {configPrintCpuTime = True}
 
+    setExpertMode :: Bool -> Config -> Config
+    setExpertMode v config = config {configExpertMode = v}
+
 printSlowItemsOption :: Option Config
 printSlowItemsOption = Option name (Just 'p') (OptArg "N" arg) "print the N slowest spec items (default: 10)" True
   where
@@ -265,7 +320,6 @@ quickCheckOptions = [
   , option "qc-max-discard" (argument "N" readMaybe setMaxDiscardRatio) "maximum number of discarded tests per successful test before giving up"
   , option "qc-max-size" (argument "N" readMaybe setMaxSize) "size to use for the biggest test cases"
   , option "qc-max-shrinks" (argument "N" readMaybe setMaxShrinks) "maximum number of shrinks to perform before giving up (a value of 0 turns shrinking off)"
-  , option "seed" (argument "N" readMaybe setSeed) "used seed for QuickCheck properties"
 
     -- for compatibility with test-framework
   , undocumented $ option "maximum-generated-tests" (argument "NUMBER" readMaybe setMaxSuccess) "how many automated tests something like QuickCheck should try, by default"
@@ -284,12 +338,13 @@ setMaxShrinks :: Int -> Config -> Config
 setMaxShrinks n c = c {configQuickCheckMaxShrinks = Just n}
 
 setSeed :: Integer -> Config -> Config
-setSeed n c = c {configQuickCheckSeed = Just n}
+setSeed n c = c {configSeed = Just n}
 
 data FailOn =
     FailOnEmpty
   | FailOnFocused
   | FailOnPending
+  | FailOnEmptyDescription
   deriving (Bounded, Enum)
 
 allFailOnItems :: [FailOn]
@@ -300,6 +355,7 @@ showFailOn item = case item of
   FailOnEmpty -> "empty"
   FailOnFocused -> "focused"
   FailOnPending -> "pending"
+  FailOnEmptyDescription -> "empty-description"
 
 readFailOn :: String -> Maybe FailOn
 readFailOn = (`lookup` items)
@@ -316,22 +372,23 @@ splitOn sep = go
 
 runnerOptions :: [Option Config]
 runnerOptions = [
-    mkFlag "dry-run" setDryRun "pretend that everything passed; don't verify anything"
-  , mkFlag "focused-only" setFocusedOnly "do not run anything, unless there are focused spec items"
+    flag "dry-run" setDryRun "pretend that everything passed; don't verify anything"
+  , flag "focused-only" setFocusedOnly "do not run anything, unless there are focused spec items"
 
-  , undocumented $ mkFlag "fail-on-focused" setFailOnFocused "fail on focused spec items"
-  , undocumented $ mkFlag "fail-on-pending" setFailOnPending "fail on pending spec items"
+  , undocumented $ flag "fail-on-focused" setFailOnFocused "fail on focused spec items"
+  , undocumented $ flag "fail-on-pending" setFailOnPending "fail on pending spec items"
 
   , mkOption    "fail-on" Nothing (argument "ITEMS" readFailOnItems (setFailOnItems True )) helpForFailOn
   , mkOption "no-fail-on" Nothing (argument "ITEMS" readFailOnItems (setFailOnItems False)) helpForFailOn
-  , mkFlag "strict" setStrict $ "same as --fail-on=" <> showFailOnItems strict
+  , flag "strict" setStrict $ "same as --fail-on=" <> showFailOnItems strict
 
-  , mkFlag "fail-fast" setFailFast "abort on first failure"
-  , mkFlag "randomize" setRandomize "randomize execution order"
-  , mkOptionNoArg "rerun" (Just 'r') setRerun "rerun all examples that failed in the previous test run (only works in combination with --failure-report or in GHCi)"
+  , flag "fail-fast" setFailFast "abort on first failure"
+  , flag "randomize" setRandomize "randomize execution order"
+  , (flag "rerun" setRerun "rerun all examples that failed in the previous test run (only works in combination with --failure-report or in GHCi)") {optionShortcut = Just 'r'}
   , option "failure-report" (argument "FILE" return setFailureReport) "read/write a failure report for use with --rerun"
-  , mkOptionNoArg "rerun-all-on-success" Nothing setRerunAllOnSuccess "run the whole test suite after a previously failing rerun succeeds for the first time (only works in combination with --rerun)"
+  , flag "rerun-all-on-success" setRerunAllOnSuccess "run the whole test suite after a previously failing rerun succeeds for the first time (only works in combination with --rerun)"
   , mkOption "jobs" (Just 'j') (argument "N" readMaxJobs setMaxJobs) "run at most N parallelizable tests simultaneously (default: number of available processors)"
+  , option "seed" (argument "N" readMaybe setSeed) "used seed for --randomize and QuickCheck properties"
   ]
   where
     strict = [FailOnFocused, FailOnPending]
@@ -347,9 +404,10 @@ runnerOptions = [
       showFailOn item <> ": " <> help item
       where
         help item = case item of
-          FailOnEmpty -> "fail if no spec items have been run"
+          FailOnEmpty -> "fail if all spec items have been filtered"
           FailOnFocused -> "fail on focused spec items"
           FailOnPending -> "fail on pending spec items"
+          FailOnEmptyDescription -> "fail on empty descriptions"
 
     setFailOnItems :: Bool -> [FailOn] -> Config -> Config
     setFailOnItems value = flip $ foldr (`setItem` value)
@@ -358,6 +416,7 @@ runnerOptions = [
           FailOnEmpty -> setFailOnEmpty
           FailOnFocused -> setFailOnFocused
           FailOnPending -> setFailOnPending
+          FailOnEmptyDescription -> setFailOnEmptyDescription
 
     readMaxJobs :: String -> Maybe Int
     readMaxJobs s = do
@@ -386,6 +445,9 @@ runnerOptions = [
     setFailOnPending :: Bool -> Config -> Config
     setFailOnPending value config = config {configFailOnPending = value}
 
+    setFailOnEmptyDescription :: Bool -> Config -> Config
+    setFailOnEmptyDescription value config = config {configFailOnEmptyDescription = value}
+
     setStrict :: Bool -> Config -> Config
     setStrict = (`setFailOnItems` strict)
 
@@ -395,8 +457,11 @@ runnerOptions = [
     setRandomize :: Bool -> Config -> Config
     setRandomize value config = config {configRandomize = value}
 
-    setRerun config = config {configRerun = True}
-    setRerunAllOnSuccess config = config {configRerunAllOnSuccess = True}
+    setRerun :: Bool -> Config -> Config
+    setRerun value config = config {configRerun = value}
+
+    setRerunAllOnSuccess :: Bool -> Config -> Config
+    setRerunAllOnSuccess value config = config {configRerunAllOnSuccess = value}
 
 commandLineOnlyOptions :: [Option Config]
 commandLineOnlyOptions = [
