@@ -19,6 +19,10 @@ module Test.Hspec.Core.Spec (
 , xdescribe
 , xcontext
 
+-- * Focused spec items #focus#
+-- |
+-- During a test run, when a spec contains /focused/ spec items, all other spec
+-- items are ignored.
 , focus
 , fit
 , fspecify
@@ -73,13 +77,15 @@ module Test.Hspec.Core.Spec (
 , Test.Hspec.Core.Tree.location
 
 , focusForest
+
+-- * Re-exports
+, HasCallStack
+, Expectation
 ) where
 
 import           Prelude ()
 import           Test.Hspec.Core.Compat
 
-import qualified Control.Exception as E
-import           Data.CallStack
 import           Control.Monad.Trans.Class (lift)
 import           Control.Monad.Trans.Reader (asks)
 
@@ -89,6 +95,7 @@ import           Test.Hspec.Core.Example
 import           Test.Hspec.Core.Hooks
 import           Test.Hspec.Core.Tree
 import           Test.Hspec.Core.Spec.Monad
+import           Test.Hspec.Core.QuickCheck ()
 
 -- | The @describe@ function combines a list of specs into a larger spec.
 describe :: HasCallStack => String -> SpecWith a -> SpecWith a
@@ -122,6 +129,16 @@ xcontext = xdescribe
 -- > describe "absolute" $ do
 -- >   it "returns a positive number when given a negative number" $
 -- >     absolute (-1) == 1
+--
+-- @`Example` a@ optionally accepts an argument @`Arg` a@, which is then given
+-- to the test body. This is useful for provisioning resources for a test which
+-- are created and cleaned up outside the test itself. See `Arg` for details.
+--
+-- Note that this function is often on the scene of nasty type errors due to GHC failing
+-- to infer the type of @do@ notation in the test body.
+-- It can be helpful to use
+-- [TypeApplications](https://ghc.gitlab.haskell.org/ghc/doc/users_guide/exts/type_applications.html)
+-- to explicitly specify the intended `Example` type.
 it :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
 it label action = fromSpecList [specItem label action]
 
@@ -146,6 +163,7 @@ xspecify = xit
 focus :: SpecWith a -> SpecWith a
 focus = mapSpecForest focusForest
 
+-- | Marks an entire spec forest as focused if nothing in it is already focused.
 focusForest :: [SpecTree a] -> [SpecTree a]
 focusForest xs
   | any (any itemIsFocused) xs = xs
@@ -188,16 +206,16 @@ setParallelizable value item = item {itemIsParallelizable = itemIsParallelizable
 -- >   it "can format text in a way that everyone likes" $
 -- >     pending
 pending :: HasCallStack => Expectation
-pending = E.throwIO (Pending location Nothing)
+pending = throwIO (Pending location Nothing)
 
 pending_ :: Expectation
-pending_ = (E.throwIO (Pending Nothing Nothing))
+pending_ = (throwIO (Pending Nothing Nothing))
 
 -- |
 -- `pendingWith` is similar to `pending`, but it takes an additional string
 -- argument that can be used to specify the reason for why the spec item is pending.
 pendingWith :: HasCallStack => String -> Expectation
-pendingWith = E.throwIO . Pending location . Just
+pendingWith = throwIO . Pending location . Just
 
 -- | Get the path of `describe` labels, from the root all the way in to the
 -- call-site of this function.
