@@ -8,14 +8,14 @@ import           Data.Char
 import           Test.Hspec.Core.Formatters.Diff as Diff
 
 dropQuotes :: String -> String
-dropQuotes = init . tail
+dropQuotes = init . drop 1
 
 spec :: Spec
 spec = do
-  describe "diff" $ do
+  describe "lineDiff" $ do
     let
       enumerate name n = map ((name ++) . show) [1 .. n :: Int]
-      diff_ expected actual = diff (Just 2) (unlines expected) (unlines actual)
+      diff_ expected actual = lineDiff (Just 2) (unlines expected) (unlines actual)
 
     it "suppresses excessive diff output" $ do
       let
@@ -23,19 +23,18 @@ spec = do
         actual = replace "foo50" "bar50" expected
 
       diff_ expected actual `shouldBe` [
-          Omitted 47
-        , Both $ unlines [
+          LinesOmitted 47
+        , LinesBoth [
             "foo48"
           , "foo49"
           ]
-        , First  "foo50"
-        , Second "bar50"
-        , Both $ unlines [
-            ""
-          , "foo51"
+        , SingleLineDiff [First "foo50", Second "bar50"]
+        , LinesBoth [
+            "foo51"
           , "foo52"
           ]
-        , Omitted 47
+        , LinesOmitted 47
+        , LinesBoth [""]
         ]
 
     it "ensures that omitted sections are at least three lines in size" $ do
@@ -43,7 +42,7 @@ spec = do
         let expected = enumerate "" size
         forAll (elements expected) $ \ i -> do
           let actual = replace i "bar" expected
-          [n | Omitted n <- diff_ expected actual] `shouldSatisfy` all (>= 3)
+          [n | LinesOmitted n <- diff_ expected actual] `shouldSatisfy` all (>= 3)
 
     context "with modifications within a line" $ do
         it "suppresses excessive diff output" $ do
@@ -52,20 +51,18 @@ spec = do
             actual = replace "foo 42" "foo 23" expected
 
           diff_ expected actual `shouldBe` [
-              Omitted 39
-            , Both $ concat [
-                "foo 40\n"
-              , "foo 41\n"
-              , "foo "
+              LinesOmitted 39
+            , LinesBoth [
+                "foo 40"
+              , "foo 41"
               ]
-            , First  "42"
-            , Second "23"
-            , Both $ concat [
-                "\n"
-              , "foo 43\n"
-              , "foo 44\n"
+            , SingleLineDiff  [Both "foo ", First "42", Second "23"]
+            , LinesBoth [
+                "foo 43"
+              , "foo 44"
               ]
-            , Omitted 55
+            , LinesOmitted 55
+            , LinesBoth [""]
             ]
 
     context "with modifications at start / end" $ do
@@ -75,23 +72,32 @@ spec = do
           actual = replace "foo9" "bar9" $ replace "foo1" "bar1" expected
 
         diff_ expected actual `shouldBe` [
-            First  "foo1"
-          , Second "bar1"
-          , Both $ unlines [
-              ""
-            , "foo2"
+            SingleLineDiff  [First "foo1", Second "bar1"]
+          , LinesBoth [
+              "foo2"
             , "foo3"
             ]
-          , Omitted 3
-          , Both $ unlines [
+          , LinesOmitted 3
+          , LinesBoth [
               "foo7"
             , "foo8"
             ]
-          , First  "foo9"
-          , Second "bar9"
-          , Both "\n"
+          , SingleLineDiff  [First "foo9", Second "bar9"]
+          , LinesBoth [""]
           ]
 
+  describe "splitLines" $ do
+    it "splits on newline characters" $ do
+      splitLines "foo\nbar\nbaz" `shouldBe` ["foo", "bar", "baz"]
+
+    describe "with a newline characters at the start" $ do
+      it "splits on newline characters" $ do
+        splitLines "\nfoo\nbar\nbaz" `shouldBe` ["", "foo", "bar", "baz"]
+
+    describe "with a newline characters at the end" $ do
+      it "splits on newline characters" $ do
+        splitLines "foo\nbar\nbaz\n" `shouldBe` ["foo", "bar", "baz", ""]
+
   describe "partition" $ do
     context "with a single shown Char" $ do
       it "never partitions a character escape" $ do
@@ -106,12 +112,12 @@ spec = do
           let
             char = dropQuotes (show [c])
             isEscaped = length char > 1
-            escape = tail char
+            escape = drop 1 char
             sep = case ys of
               x : _ | all isDigit escape && isDigit x || escape == "SO" && x == 'H' -> ["\\&"]
               _ -> []
             actual = partition (show (xs ++ c : ys))
-            expected = partition (init $ show xs) ++ [char] ++ sep ++ partition (tail $ show ys)
+            expected = partition (init $ show xs) ++ [char] ++ sep ++ partition (drop 1 $ show ys)
           in isEscaped ==> actual `shouldBe` expected
 
   describe "breakList" $ do
